package wechat

import (
	"context"
	"encoding/json"
	//"dc_golang_server_1/conf"
	dao "dc_golang_server_1/dao/base"
	"dc_golang_server_1/exception"
	"dc_golang_server_1/model/wechat/params"
	"dc_golang_server_1/model/wechat/response"
	"dc_golang_server_1/util"
	"github.com/wechatpay-apiv3/wechatpay-go/services/payments"
	"github.com/wechatpay-apiv3/wechatpay-go/services/refunddomestic"
	"io/ioutil"
	"math"
	"net/http"
	"strconv"
	"strings"
	"time"

	"github.com/wechatpay-apiv3/wechatpay-go/core"
	"github.com/wechatpay-apiv3/wechatpay-go/services/partnerpayments/jsapi"
)

const MainHost = "" //回调通知的外网域名

func ToWxMoney(amount float64) *int64 {
	return core.Int64(int64(math.Floor(amount * 100)))
}

//JsapiPartnerPrepayOrder 服务商模式小程序支付
func JsapiPartnerPrepayOrder(params params.WechatPartnerJsapiOrderParams) response.WxMiniRequestPayment {
	wechatPayConfig, err := dao.FindWechatPayConfigByPrimaryKey(*params.AreaServiceId)
	if wechatPayConfig.AreaServiceId == nil {
		exception.ThrowsErrS("[" + strconv.Itoa(int(*params.AreaServiceId)) + "]对应微信支付配不存在")
	}
	wechatClient := BuildClient(*wechatPayConfig.SpMchid)
	svc := jsapi.JsapiApiService{Client: wechatClient.Client}
	// 得到prepay_id，以及调起支付所需的参数和签名
	resp, _, err := svc.Prepay(context.Background(),
		jsapi.PrepayRequest{
			SpAppid:     core.String(*wechatPayConfig.SpAppid),
			SpMchid:     core.String(*wechatPayConfig.SpMchid),
			SubMchid:    core.String(*wechatPayConfig.SubMchid),
			Description: core.String(*params.Title),
			OutTradeNo:  core.String(*params.OutTradeNo),
			NotifyUrl:   core.String(MainHost + "/wechat/partner/pay/notify/" + *wechatPayConfig.SpMchid),
			Amount: &jsapi.Amount{
				Total: ToWxMoney(*params.Amount),
			},
			Payer: &jsapi.Payer{
				SpOpenid: params.OpenId,
			},
		},
	)

	if err != nil {
		exception.ThrowsErr(err)
	}
	return buildWxMiniPayInfo(*wechatPayConfig.SpAppid, *resp.PrepayId, wechatClient)
}

func buildWxMiniPayInfo(appId string, prepayId string, wechatClient WxClient) response.WxMiniRequestPayment {
	var rd response.WxMiniRequestPayment
	rd.AppId = appId
	rd.TimeStamp = util.Strval(time.Now().Unix())
	rd.NonceStr = util.FandomString(32)
	rd.Package = "prepay_id=" + prepayId
	rd.SignType = "RSA"
	paySign := SignPay(rd.AppId, rd.TimeStamp, rd.NonceStr, rd.Package, wechatClient.PrivateKey)
	rd.PaySign = paySign
	return rd
}

func PayBack(mchID string, request *http.Request) bool {
	handler := GetHandler(mchID)
	transaction := new(payments.Transaction)
	notifyReq, err := handler.ParseNotifyRequest(context.Background(), request, transaction)
	if err != nil {
		exception.ThrowsErr(err)
	}
	if notifyReq != nil {
		if *transaction.TradeState == "SUCCESS" {
			payBackBusiness(*transaction.OutTradeNo)
			return true
		}
	}
	return false
}

/**
支付回调处理业务
注意：
同样的通知可能会多次发送给商户系统。商户系统必须能够正确处理重复的通知。 推荐的做法是，当商户系统收到通知进行处理时，先检查对应业务数据的状态，并判断该通知是否已经处理。如果未处理，则再进行处理；如果已处理，则直接返回结果成功。在对业务数据进行状态检查和处理之前，要采用数据锁进行并发控制，以避免函数重入造成的数据混乱。
*/
func payBackBusiness(outTradeNo string) {

}

type RefundNotifyData struct {
	SpMchId             *string                 `json:"sp_mchid,omitempty"`
	SubMchid            *string                 `json:"sub_mchid,omitempty"`
	TransactionId       *string                 `json:"transaction_id,omitempty"`
	OutTradeNo          *string                 `json:"out_trade_no,omitempty"`
	RefundId            *string                 `json:"refund_id,omitempty"`
	OutRefundNo         *string                 `json:"out_refund_no,omitempty"`
	RefundStatus        *string                 `json:"refund_status,omitempty"`
	SuccessTime         *string                 `json:"success_time,omitempty"`
	UserReceivedAccount string                  `json:"user_received_account,omitempty"`
	Amount              *RefundNotifyAmountData `json:"amount"`
}

type RefundNotifyAmountData struct {
	Total       *int64 `json:"total,omitempty"`
	Refund      *int64 `json:"refund,omitempty"`
	PayerTotal  *int64 `json:"payer_total,omitempty"`
	PayerRefund *int64 `json:"payer_refund,omitempty"`
}

func RefundBack(mchID string, request *http.Request) bool {
	handler := GetHandler(mchID)
	refund := new(RefundNotifyData)
	notifyReq, err := handler.ParseNotifyRequest(context.Background(), request, refund)
	if err != nil {
		exception.ThrowsErr(err)
		return false
	}
	if notifyReq != nil {
		if *refund.RefundStatus == "SUCCESS" {
			refundBackBusiness(*refund.OutRefundNo)
		}

	}
	return false
}

/**
退款回调处理业务
注意：
同样的通知可能会多次发送给商户系统。商户系统必须能够正确处理重复的通知。 推荐的做法是，当商户系统收到通知进行处理时，先检查对应业务数据的状态，并判断该通知是否已经处理。如果未处理，则再进行处理；如果已处理，则直接返回结果成功。在对业务数据进行状态检查和处理之前，要采用数据锁进行并发控制，以避免函数重入造成的数据混乱。
*/
func refundBackBusiness(outRefundNo string) {

}

/** Refund
退款
*/
func Refund(params params.WechatPartnerRefundParams) {
	wechatPayConfig, err := dao.FindWechatPayConfigByPrimaryKey(*params.AreaServiceId)
	if wechatPayConfig.AreaServiceId == nil {
		exception.ThrowsErrS("[" + strconv.Itoa(int(*params.AreaServiceId)) + "]对应微信支付配不存在")
	}
	wechatClient := BuildClient(*wechatPayConfig.SpMchid)
	svc := refunddomestic.RefundsApiService{Client: wechatClient.Client}
	resp, _, err := svc.Create(context.Background(), refunddomestic.CreateRequest{
		SubMchid:    wechatPayConfig.SubMchid,
		OutTradeNo:  params.OutTradeNo,
		OutRefundNo: params.OutRefundNo,
		NotifyUrl:   core.String(MainHost + "/wechat/partner/refund/notify/" + *wechatPayConfig.SpMchid),
		Amount: &refunddomestic.AmountReq{
			Refund:   ToWxMoney(*params.RefundAmount),
			Total:    ToWxMoney(*params.OrderAmount),
			Currency: core.String("CNY"),
		},
	})
	if err != nil {
		var apiErr = err.(*core.APIError)
		exception.ThrowsErrS(apiErr.Message)
	}
	if *resp.Status != refunddomestic.STATUS_SUCCESS &&
		*resp.Status != refunddomestic.STATUS_PROCESSING {
		exception.ThrowsErrS("退款错误")
	}
}

func JsCode2session(code string, appId string) (responseData response.WxOAuth2ResponseData, err error) {
	//var NAME = "微信授权"
	//var name = NAME + "-code2Session"
	var url = "https://web.weixin.qq.com/sns/jscode2session?appid=${appId}&secret=${secret}&js_code=${code}&grant_type=authorization_code"
	baseWechatPayConfig, err := dao.FindWechatAppConfigByPrimaryKey(appId)
	url = strings.ReplaceAll(url, "${appId}", *baseWechatPayConfig.AppId)
	url = strings.ReplaceAll(url, "${secret}", *baseWechatPayConfig.Secret)
	url = strings.ReplaceAll(url, "${code}", code)
	//logger.Log.Debug(name,
	//	zap.String("url", url))
	request, err := http.NewRequest(http.MethodGet, url, nil)
	client := http.DefaultClient
	response, err := client.Do(request)
	if err != nil {
		return responseData, err
	}
	defer func() {
		response.Body.Close()
	}()
	bodyBytes, err := ioutil.ReadAll(response.Body)
	//logger.Log.Debug(name,
	//	zap.String("url", url),
	//	zap.String("response", string(bodyBytes)))
	if err != nil {
		return responseData, err
	}
	if err = json.Unmarshal(bodyBytes, &responseData); err != nil {
		return responseData, err
	}
	if responseData.Errmsg != "" {
		exception.ThrowsErrS(responseData.Errmsg)
	}
	return responseData, nil
}
